package cn.com.infosec.crypto.modes;

import cn.com.infosec.crypto.BlockCipher;
import cn.com.infosec.crypto.CipherParameters;
import cn.com.infosec.crypto.InvalidCipherTextException;
import cn.com.infosec.crypto.macs.CBCBlockCipherMac;
import cn.com.infosec.crypto.params.AEADParameters;
import cn.com.infosec.crypto.params.ParametersWithIV;
import cn.com.infosec.util.Arrays;
import java.io.ByteArrayOutputStream;

public class CCMBlockCipher implements AEADBlockCipher {
   private BlockCipher cipher;
   private int blockSize;
   private boolean forEncryption;
   private byte[] nonce;
   private byte[] associatedText;
   private int macSize;
   private CipherParameters keyParam;
   private byte[] macBlock;
   private ByteArrayOutputStream data = new ByteArrayOutputStream();

   public CCMBlockCipher(BlockCipher c) {
      this.cipher = c;
      int c1;
      int var10000 = c1 = c.getBlockSize();
      this.blockSize = c1;
      this.macBlock = new byte[c1];
      if (var10000 != 16) {
         throw new IllegalArgumentException("cipher required with a block size of 16.");
      }
   }

   private int calculateMac(byte[] data, int dataOff, int dataLen, byte[] macBlock) {
      CBCBlockCipherMac var5;
      CBCBlockCipherMac var10001 = var5 = new CBCBlockCipherMac;
      var5.<init>(this.cipher, this.macSize * 8);
      var10001.init(this.keyParam);
      int var6 = 16;
      byte[] var7 = new byte[16];
      if (this.hasAssociatedText()) {
         var7[0] = (byte)(var7[0] | 64);
      }

      var7[0] = (byte)(var7[0] | ((var5.getMacSize() - 2) / 2 & 7) << 3);
      int var8 = var7[0];
      byte[] var9;
      byte[] var10000 = var9 = this.nonce;
      var7[0] = (byte)(var8 | 15 - var9.length - 1 & 7);
      var8 = var10000.length;
      System.arraycopy(var10000, 0, var7, 1, var8);
      var8 = 1;

      for(int var12 = dataLen; var12 > 0; ++var8) {
         int var13 = var12;
         int var10002 = var12;
         var12 = var6 - var8;
         var7[var12] = (byte)(var10002 & 255);
         var12 = var13 >>> 8;
      }

      var5.update(var7, 0, var6);
      if (this.hasAssociatedText()) {
         byte[] var11;
         byte var14;
         if ((var11 = this.associatedText).length < 65280) {
            var5.update((byte)(var11.length >> 8));
            var5.update((byte)this.associatedText.length);
            var14 = 2;
         } else {
            var5.update((byte)-1);
            var5.update((byte)-2);
            var5.update((byte)(this.associatedText.length >> 24));
            var5.update((byte)(this.associatedText.length >> 16));
            var5.update((byte)(this.associatedText.length >> 8));
            var5.update((byte)this.associatedText.length);
            var14 = 6;
         }

         byte[] var10003 = this.associatedText;
         var5.update(var10003, 0, var10003.length);
         int var10;
         if ((var10 = (var14 + this.associatedText.length) % 16) != 0) {
            for(var6 = 0; var6 != 16 - var10; ++var6) {
               var5.update((byte)0);
            }
         }
      }

      var5.update(data, dataOff, dataLen);
      return var5.doFinal(macBlock, 0);
   }

   private boolean hasAssociatedText() {
      byte[] var1;
      return (var1 = this.associatedText) != null && var1.length != 0;
   }

   public BlockCipher getUnderlyingCipher() {
      return this.cipher;
   }

   public void init(boolean forEncryption, CipherParameters params) {
      this.forEncryption = forEncryption;
      CCMBlockCipher var10000;
      CCMBlockCipher var10001;
      CCMBlockCipher var10002;
      if (params instanceof AEADParameters) {
         var10000 = this;
         var10001 = this;
         var10002 = this;
         AEADParameters var3;
         this.nonce = (var3 = (AEADParameters)params).getNonce();
         var10002.associatedText = var3.getAssociatedText();
         var10001.macSize = var3.getMacSize() / 8;
         var10000.keyParam = var3.getKey();
      } else {
         if (!(params instanceof ParametersWithIV)) {
            throw new IllegalArgumentException("invalid parameters passed to CCM");
         }

         var10000 = this;
         var10001 = this;
         var10002 = this;
         CCMBlockCipher var10003 = this;
         ParametersWithIV var4;
         this.nonce = (var4 = (ParametersWithIV)params).getIV();
         var10003.associatedText = null;
         var10001.macSize = var10002.macBlock.length / 2;
         var10000.keyParam = var4.getParameters();
      }

   }

   public String getAlgorithmName() {
      return this.cipher.getAlgorithmName() + "/CCM";
   }

   public int processByte(byte in, byte[] out, int outOff) {
      this.data.write(in);
      return 0;
   }

   public int processBytes(byte[] in, int inOff, int inLen, byte[] out, int outOff) {
      this.data.write(in, inOff, inLen);
      return 0;
   }

   public int doFinal(byte[] out, int outOff) {
      CCMBlockCipher var10000 = this;
      byte[] var10002 = this.data.toByteArray();
      byte[] var4;
      byte[] var10001 = var4 = this.processPacket(var10002, 0, var10002.length);
      int var3 = var10001.length;
      System.arraycopy(var10001, 0, out, outOff, var3);
      var10000.reset();
      return var4.length;
   }

   public void reset() {
      this.cipher.reset();
      this.data.reset();
   }

   public byte[] getMac() {
      int var1;
      byte[] var2;
      byte[] var10000 = var2 = new byte[var1 = this.macSize];
      System.arraycopy(this.macBlock, 0, var2, 0, var1);
      return var10000;
   }

   public int getUpdateOutputSize(int len) {
      return 0;
   }

   public int getOutputSize(int len) {
      return this.forEncryption ? this.data.size() + len + this.macSize : this.data.size() + len - this.macSize;
   }

   public byte[] processPacket(byte[] in, int inOff, int inLen) {
      if (this.keyParam == null) {
         throw new IllegalStateException("CCM cipher unitialized.");
      } else {
         SICBlockCipher var4;
         SICBlockCipher var10001 = var4 = new SICBlockCipher;
         var4.<init>(this.cipher);
         byte[] var5 = new byte[this.blockSize];
         byte[] var10003 = this.nonce;
         var5[0] = (byte)(15 - var10003.length - 1 & 7);
         int var6 = var10003.length;
         System.arraycopy(var10003, 0, var5, 1, var6);
         var10001.init(this.forEncryption, new ParametersWithIV(this.keyParam, var5));
         byte[] var7;
         byte[] var8;
         byte[] var9;
         int in1;
         int var12;
         int var13;
         int var10000;
         byte[] var10002;
         if (this.forEncryption) {
            var12 = 0;
            var7 = new byte[var6 = inLen + this.macSize];
            var8 = this.macBlock;
            this.calculateMac(in, inOff, inLen, var8);
            byte[] var16 = this.macBlock;
            var4.processBlock(var16, 0, var16, 0);

            while(inOff < inLen - (var13 = this.blockSize)) {
               var10000 = inOff;
               var4.processBlock(in, inOff, var7, var12);
               var12 += inOff = this.blockSize;
               inOff += var10000;
            }

            var10002 = var9 = new byte[var13];
            in1 = inLen - inOff;
            System.arraycopy(in, inOff, var9, 0, in1);
            var4.processBlock(var9, 0, var9, 0);
            System.arraycopy(var10002, 0, var7, var12, in1);
            int var10 = var12 + in1;
            in1 = var6 - var10;
            System.arraycopy(this.macBlock, 0, var7, var10, in1);
         } else {
            int var10006 = inLen;
            int var10007 = inLen;
            inLen = 0;
            var7 = new byte[var6 = var10007 - (var12 = this.macSize)];
            var13 = inOff + var10006 - var12;
            System.arraycopy(in, var13, this.macBlock, 0, var12);
            var10002 = this.macBlock;
            var4.processBlock(var10002, 0, var10002, 0);

            for(var12 = this.macSize; var12 != (var8 = this.macBlock).length; ++var12) {
               var8[var12] = 0;
            }

            while(inLen < var6 - (var12 = this.blockSize)) {
               var10000 = inOff;
               var4.processBlock(in, inOff, var7, inLen);
               inLen += inOff = this.blockSize;
               inOff += var10000;
            }

            byte[] var10004 = var9 = new byte[var12];
            in1 = var6 - inLen;
            System.arraycopy(in, inOff, var9, 0, in1);
            var4.processBlock(var9, 0, var9, 0);
            System.arraycopy(var10004, 0, var7, inLen, in1);
            var9 = new byte[this.blockSize];
            this.calculateMac(var7, 0, var6, var9);
            if (!Arrays.areEqual(this.macBlock, var9)) {
               throw new InvalidCipherTextException("mac check in CCM failed");
            }
         }

         return var7;
      }
   }
}
